@@ -4,9 +4,11 @@ This is designed to be the root segment, without
 any children, and the output of the lexer.
 """
 
-from typing import Any, FrozenSet, List, Optional, Tuple
+from typing import Any, Callable, Optional, Union, cast
 from uuid import uuid4
 
+import regex as re
+
 from sqlfluff.core.parser.markers import PositionMarker
 from sqlfluff.core.parser.segments.base import BaseSegment, SourceFix
 
@@ -30,11 +32,14 @@ class RawSegment(BaseSegment):
         # `type` argument here, but for more precise inheritance
         # we suggest using the `instance_types` option.
         type: Optional[str] = None,
-        instance_types: Tuple[str, ...] = (),
-        trim_start: Optional[Tuple[str, ...]] = None,
-        trim_chars: Optional[Tuple[str, ...]] = None,
-        source_fixes: Optional[List[SourceFix]] = None,
+        instance_types: tuple[str, ...] = (),
+        trim_start: Optional[tuple[str, ...]] = None,
+        trim_chars: Optional[tuple[str, ...]] = None,
+        source_fixes: Optional[list[SourceFix]] = None,
         uuid: Optional[int] = None,
+        quoted_value: Optional[tuple[str, Union[int, str]]] = None,
+        escape_replacements: Optional[list[tuple[str, str]]] = None,
+        casefold: Optional[Callable[[str], str]] = None,
     ):
         """Initialise raw segment.
 
@@ -53,7 +58,7 @@ class RawSegment(BaseSegment):
         self.pos_marker: PositionMarker = pos_marker  # type: ignore
         # Set the segments attribute to be an empty tuple.
         self.segments = ()
-        self.instance_types: Tuple[str, ...]
+        self.instance_types: tuple[str, ...]
         if type:
             assert not instance_types, "Cannot set `type` and `instance_types`."
             self.instance_types = (type,)
@@ -69,6 +74,10 @@ class RawSegment(BaseSegment):
         self.representation = "<{}: ({}) {!r}>".format(
             self.__class__.__name__, self.pos_marker, self.raw
         )
+        self.quoted_value = quoted_value
+        self.escape_replacements = escape_replacements
+        self.casefold = casefold
+        self._raw_value: str = self._raw_normalized()
 
     def __repr__(self) -> str:
         # This is calculated at __init__, because all elements are immutable
@@ -108,12 +117,12 @@ class RawSegment(BaseSegment):
         return self._raw_upper
 
     @property
-    def raw_segments(self) -> List["RawSegment"]:
+    def raw_segments(self) -> list["RawSegment"]:
         """Returns self to be compatible with calls to its superclass."""
         return [self]
 
     @property
-    def class_types(self) -> FrozenSet[str]:
+    def class_types(self) -> frozenset[str]:
         """The set of full types for this segment, including inherited.
 
         Add the surrogate type for raw segments.
@@ -121,7 +130,7 @@ class RawSegment(BaseSegment):
         return frozenset(self.instance_types) | super().class_types
 
     @property
-    def source_fixes(self) -> List[SourceFix]:
+    def source_fixes(self) -> list[SourceFix]:
         """Return any source fixes as list."""
         return self._source_fixes or []
 
@@ -143,7 +152,7 @@ class RawSegment(BaseSegment):
             return True
         return self.class_is_type(*seg_type)
 
-    def get_raw_segments(self) -> List["RawSegment"]:
+    def get_raw_segments(self) -> list["RawSegment"]:
         """Iterate raw segments, mostly for searching."""
         return [self]
 
@@ -171,6 +180,40 @@ class RawSegment(BaseSegment):
             return raw_buff
         return raw_buff
 
+    def _raw_normalized(self) -> str:
+        """Returns the string of the raw content's value.
+
+        E.g. This removes leading and trailing quote characters, removes escapes
+
+        Return:
+        str: The raw content's value
+        """
+        raw_buff = self.raw
+        if self.quoted_value:
+            _match = re.match(self.quoted_value[0], raw_buff)
+            if _match:
+                _group_match = _match.group(self.quoted_value[1])
+                if isinstance(_group_match, str):
+                    raw_buff = _group_match
+        if self.escape_replacements:
+            for old, new in self.escape_replacements:
+                raw_buff = re.sub(old, new, raw_buff)
+        return raw_buff
+
+    def raw_normalized(self, casefold: bool = True) -> str:
+        """Returns a normalized string of the raw content.
+
+        E.g. This removes leading and trailing quote characters, removes escapes,
+        optionally casefolds to the dialect's casing
+
+        Return:
+        str: The normalized version of the raw content
+        """
+        raw_buff = self._raw_value
+        if self.casefold and casefold:
+            raw_buff = self.casefold(raw_buff)
+        return raw_buff
+
     def stringify(
         self, ident: int = 0, tabsize: int = 4, code_only: bool = False
     ) -> str:
@@ -199,13 +242,13 @@ class RawSegment(BaseSegment):
         return f"{self.raw!r}"
 
     def edit(
-        self, raw: Optional[str] = None, source_fixes: Optional[List[SourceFix]] = None
+        self, raw: Optional[str] = None, source_fixes: Optional[list[SourceFix]] = None
     ) -> "RawSegment":
         """Create a new segment, with exactly the same position but different content.
 
         Args:
             raw (Optional[str]): The new content for the segment.
-            source_fixes (Optional[List[SourceFix]]): A list of fixes to be applied to
+            source_fixes (Optional[list[SourceFix]]): A list of fixes to be applied to
                 the segment.
 
         Returns:
@@ -223,9 +266,38 @@ class RawSegment(BaseSegment):
             instance_types=self.instance_types,
             trim_start=self.trim_start,
             trim_chars=self.trim_chars,
+            quoted_value=self.quoted_value,
+            escape_replacements=self.escape_replacements,
+            casefold=self.casefold,
             source_fixes=source_fixes or self.source_fixes,
         )
 
+    def _get_raw_segment_kwargs(self) -> dict[str, Any]:
+        return {
+            "quoted_value": self.quoted_value,
+            "escape_replacements": self.escape_replacements,
+            "casefold": self.casefold,
+        }
+
+    # ################ CLASS METHODS
+
+    @classmethod
+    def from_result_segments(
+        cls,
+        result_segments: tuple[BaseSegment, ...],
+        segment_kwargs: dict[str, Any],
+    ) -> "RawSegment":
+        """Create a RawSegment from result segments."""
+        assert len(result_segments) == 1
+        raw_seg = cast("RawSegment", result_segments[0])
+        new_segment_kwargs = raw_seg._get_raw_segment_kwargs()
+        new_segment_kwargs.update(segment_kwargs)
+        return cls(
+            raw=raw_seg.raw,
+            pos_marker=raw_seg.pos_marker,
+            **new_segment_kwargs,
+        )
+
 
 __all__ = [
     "PositionMarker",
