@@ -13,6 +13,7 @@ from sqlfluff.core.parser import (
     Dedent,
     Delimited,
     IdentifierSegment,
+    ImplicitIndent,
     Indent,
     Matchable,
     Nothing,
@@ -62,6 +63,7 @@ duckdb_dialect.sets("unreserved_keywords").update(
         "ANTI",
         "ASOF",
         "MACRO",
+        "MAP",
         "POSITIONAL",
         "SEMI",
         "STRUCT",
@@ -71,6 +73,7 @@ duckdb_dialect.sets("unreserved_keywords").update(
 
 duckdb_dialect.add(
     LambdaArrowSegment=StringParser("->", SymbolSegment, type="lambda_arrow"),
+    OrIgnoreGrammar=Sequence("OR", "IGNORE"),
 )
 
 duckdb_dialect.replace(
@@ -205,6 +208,79 @@ class StructTypeSchemaSegment(BaseSegment):
     )
 
 
+class MapTypeSegment(ansi.MapTypeSegment):
+    """Expression to construct a MAP datatype."""
+
+    match_grammar = Sequence(
+        "MAP",
+        Ref("MapTypeSchemaSegment", optional=True),
+    )
+
+
+class MapTypeSchemaSegment(BaseSegment):
+    """Expression to construct the schema of a MAP datatype."""
+
+    type = "map_type_schema"
+    match_grammar = Bracketed(
+        Delimited(
+            Ref("DatatypeSegment"),
+        ),
+    )
+
+
+class InsertStatementSegment(ansi.InsertStatementSegment):
+    """An `INSERT` Statement.
+
+    https://duckdb.org/docs/stable/sql/statements/insert.html
+    """
+
+    type = "insert_statement"
+    match_grammar: Matchable = Sequence(
+        "INSERT",
+        OneOf(Ref("OrReplaceGrammar"), Ref("OrIgnoreGrammar"), optional=True),
+        "INTO",
+        Ref("TableReferenceSegment"),
+        Ref("AsAliasExpressionSegment", optional=True),
+        OneOf(
+            Ref("BracketedColumnReferenceListGrammar"),
+            Sequence("BY", "POSITION"),
+            Sequence("BY", "NAME"),
+            optional=True,
+        ),
+        OneOf(
+            Sequence("DEFAULT", "VALUES"),
+            Ref("SelectStatementSegment"),
+            Sequence(
+                Ref("BracketedColumnReferenceListGrammar", optional=True),
+                OneOf(
+                    Ref("ValuesClauseSegment"),
+                    OptionallyBracketed(Ref("SelectStatementSegment")),
+                ),
+            ),
+        ),
+        Sequence(
+            "ON",
+            "CONFLICT",
+            Ref("ConflictTargetSegment", optional=True),
+            Ref("ConflictActionSegment"),
+            optional=True,
+        ),
+        Sequence(
+            "RETURNING",
+            OneOf(
+                Ref("StarSegment"),
+                Delimited(
+                    Sequence(
+                        Ref("ExpressionSegment"),
+                        Ref("AsAliasExpressionSegment", optional=True),
+                    ),
+                ),
+            ),
+            optional=True,
+        ),
+    )
+
+
 class ColumnConstraintSegment(ansi.ColumnConstraintSegment):
     """A column option; each CREATE TABLE column can have 0 or more.
 
@@ -427,6 +503,16 @@ class SelectStatementSegment(ansi.SelectStatementSegment):
     match_grammar = ansi.SelectStatementSegment.match_grammar.copy(
         insert=[Ref("QualifyClauseSegment", optional=True)],
         before=Ref("OrderByClauseSegment", optional=True),
+        replace_terminators=True,
+        terminators=[
+            Ref("SetOperatorSegment"),
+            Ref("WithNoSchemaBindingClauseSegment"),
+            Ref("WithDataClauseSegment"),
+            Sequence("ON", "CONFLICT"),
+            "RETURNING",
+            Ref("WithCheckOptionSegment"),
+            Ref("MetaCommandQueryBufferSegment"),
+        ],
     )
 
 
@@ -462,6 +548,8 @@ class UnorderedSelectStatementSegment(ansi.UnorderedSelectStatementSegment):
             Ref("SetOperatorSegment"),
             Ref("OrderByClauseSegment"),
             Ref("LimitClauseSegment"),
+            Sequence("ON", "CONFLICT"),
+            "RETURNING",
         ],
     )
 
@@ -541,7 +629,7 @@ class QualifyClauseSegment(BaseSegment):
     type = "qualify_clause"
     match_grammar = Sequence(
         "QUALIFY",
-        Indent,
+        ImplicitIndent,
         OptionallyBracketed(Ref("ExpressionSegment")),
         Dedent,
     )
