@@ -9,10 +9,8 @@ There are also some keywords that are(n't) supported as types and function, but
 isn't support for that distinction at present.
 """
 
-from typing import List, Tuple
 
-
-def priority_keyword_merge(*args: List[Tuple[str, str]]) -> List[Tuple[str, str]]:
+def priority_keyword_merge(*args: list[tuple[str, str]]) -> list[tuple[str, str]]:
     """Merge keyword lists, giving priority to entries in later lists.
 
     *args is a list of keyword lists, these lists should be of tuples in the form
@@ -26,7 +24,7 @@ def priority_keyword_merge(*args: List[Tuple[str, str]]) -> List[Tuple[str, str]
 
     while len(keyword_lists) > 1:
         base_list, priority_list = keyword_lists[0], keyword_lists[1]
-        keyword_set = set([x[0] for x in base_list])
+        keyword_set = {x[0] for x in base_list}
         for item in priority_list:
             if item[0] in keyword_set:
                 for index, keyword in enumerate(base_list):
@@ -40,7 +38,7 @@ def priority_keyword_merge(*args: List[Tuple[str, str]]) -> List[Tuple[str, str]
     return base_list
 
 
-def get_keywords(keyword_list: List[Tuple[str, str]], keyword_type: str) -> List[str]:
+def get_keywords(keyword_list: list[tuple[str, str]], keyword_type: str) -> list[str]:
     """Get a list of keywords of the required type.
 
     keyword_type should be one of "not-keyword", "reserved", "non-reserved"
@@ -168,6 +166,7 @@ postgres_docs_keywords = [
     ("COMMENTS", "non-reserved"),
     ("COMMIT", "non-reserved"),
     ("COMMITTED", "non-reserved"),
+    ("COMMUTATOR", "reserved-(can-be-function-or-type)"),
     ("COMPRESSION", "non-reserved"),
     ("CONCURRENTLY", "reserved-(can-be-function-or-type)"),
     ("CONDITION", "not-keyword"),
@@ -357,6 +356,7 @@ postgres_docs_keywords = [
     ("GROUPING", "non-reserved-(cannot-be-function-or-type)"),
     ("GROUPS", "non-reserved"),
     ("HANDLER", "non-reserved"),
+    ("HASHES", "reserved-(can-be-function-or-type)"),
     ("HAVING", "reserved"),
     ("HEADER", "non-reserved"),
     ("HEX", "not-keyword"),
@@ -411,6 +411,7 @@ postgres_docs_keywords = [
     ("INVOKER", "non-reserved"),
     ("IS", "reserved-(can-be-function-or-type)"),
     ("ISNULL", "reserved-(can-be-function-or-type)"),
+    ("IS_SUPERUSER", "non-reserved"),
     ("ISOLATION", "non-reserved"),
     ("JOIN", "reserved-(can-be-function-or-type)"),
     ("JSON", "not-keyword"),
@@ -437,11 +438,14 @@ postgres_docs_keywords = [
     ("LAST", "non-reserved"),
     ("LAST_VALUE", "not-keyword"),
     ("LATERAL", "reserved"),
+    ("LC_COLLATE", "non-reserved"),
+    ("LC_CTYPE", "non-reserved"),
     ("LEAD", "not-keyword"),
     ("LEADING", "reserved"),
     ("LEAKPROOF", "non-reserved"),
     ("LEAST", "non-reserved-(cannot-be-function-or-type)"),
     ("LEFT", "reserved-(can-be-function-or-type)"),
+    ("LEFTARG", "non-reserved-(cannot-be-function-or-type)"),
     ("LENGTH", "not-keyword"),
     ("LEVEL", "non-reserved"),
     ("LIBRARY", "not-keyword"),
@@ -482,6 +486,7 @@ postgres_docs_keywords = [
     ("MEASURES", "not-keyword"),
     ("MEMBER", "not-keyword"),
     ("MERGE", "non-reserved"),
+    ("MERGES", "reserved-(can-be-function-or-type)"),
     ("MESSAGE_LENGTH", "not-keyword"),
     ("MESSAGE_OCTET_LENGTH", "not-keyword"),
     ("MESSAGE_TEXT", "not-keyword"),
@@ -506,6 +511,7 @@ postgres_docs_keywords = [
     ("NATURAL", "reserved-(can-be-function-or-type)"),
     ("NCHAR", "non-reserved-(cannot-be-function-or-type)"),
     ("NCLOB", "not-keyword"),
+    ("NEGATOR", "reserved-(can-be-function-or-type)"),
     ("NESTED", "not-keyword"),
     ("NESTING", "not-keyword"),
     ("NEW", "non-reserved"),
@@ -674,6 +680,7 @@ postgres_docs_keywords = [
     ("RETURNS", "non-reserved"),
     ("REVOKE", "non-reserved"),
     ("RIGHT", "reserved-(can-be-function-or-type)"),
+    ("RIGHTARG", "reserved-(can-be-function-or-type)"),
     ("ROLE", "non-reserved"),
     ("ROLLBACK", "non-reserved"),
     ("ROLLUP", "non-reserved"),
@@ -716,7 +723,9 @@ postgres_docs_keywords = [
     ("SERIAL8", "non-reserved-(cannot-be-function-or-type)"),
     ("SERIALIZABLE", "non-reserved"),
     ("SERVER", "non-reserved"),
+    ("SERVER_ENCODING", "non-reserved"),
     ("SERVER_NAME", "not-keyword"),
+    ("SERVER_VERSION", "non-reserved"),
     ("SESSION", "non-reserved"),
     ("SESSION_USER", "reserved"),
     ("SET", "non-reserved"),
@@ -847,7 +856,7 @@ postgres_docs_keywords = [
     ("UPPER", "not-keyword"),
     ("URI", "not-keyword"),
     ("USAGE", "not-keyword"),
-    ("USER", "reserved"),
+    ("USER", "non-reserved"),
     ("USER_DEFINED_TYPE_CATALOG", "not-keyword"),
     ("USER_DEFINED_TYPE_CODE", "not-keyword"),
     ("USER_DEFINED_TYPE_NAME", "not-keyword"),
