@@ -1,8 +1,9 @@
 """Dataclasses for reflow work."""
 
 import logging
+from collections.abc import Iterator, Sequence
 from itertools import chain
-from typing import Iterator, List, Optional, Sequence, Tuple, Type, cast
+from typing import Literal, Optional, cast
 
 from sqlfluff.core.config import FluffConfig
 from sqlfluff.core.parser import BaseSegment, RawSegment
@@ -16,7 +17,7 @@ from sqlfluff.utils.reflow.elements import (
     get_consumed_whitespace,
 )
 from sqlfluff.utils.reflow.helpers import fixes_from_results
-from sqlfluff.utils.reflow.rebreak import rebreak_sequence
+from sqlfluff.utils.reflow.rebreak import rebreak_keywords_sequence, rebreak_sequence
 from sqlfluff.utils.reflow.reindent import (
     construct_single_indent,
     lint_indent_points,
@@ -65,7 +66,7 @@ class ReflowSequence:
         root_segment: BaseSegment,
         reflow_config: ReflowConfig,
         depth_map: DepthMap,
-        lint_results: Optional[List[LintResult]] = None,
+        lint_results: Optional[list[LintResult]] = None,
     ):
         # First validate integrity
         self._validate_reflow_sequence(elements)
@@ -81,9 +82,9 @@ class ReflowSequence:
         # Rather than saving *fixes* directly, we package them into
         # LintResult objects to make it a little easier to expose them
         # in the CLI.
-        self.lint_results: List[LintResult] = lint_results or []
+        self.lint_results: list[LintResult] = lint_results or []
 
-    def get_fixes(self) -> List[LintFix]:
+    def get_fixes(self) -> list[LintFix]:
         """Get the current fix buffer.
 
         We're hydrating them here directly from the LintResult
@@ -93,7 +94,7 @@ class ReflowSequence:
         """
         return fixes_from_results(self.lint_results)
 
-    def get_results(self) -> List[LintResult]:
+    def get_results(self) -> list[LintResult]:
         """Return the current result buffer."""
         return self.lint_results
 
@@ -136,7 +137,7 @@ class ReflowSequence:
         which simplifies iteration here.
         """
         elem_buff: ReflowSequenceType = []
-        seg_buff: List[RawSegment] = []
+        seg_buff: list[RawSegment] = []
         for seg in segments:
             # NOTE: end_of_file is block-like rather than point-like.
             # This is to facilitate better evaluation of the ends of files.
@@ -156,7 +157,7 @@ class ReflowSequence:
             # Add the block, with config info.
             elem_buff.append(
                 ReflowBlock.from_config(
-                    segments=[seg],
+                    segments=(seg,),
                     config=reflow_config,
                     depth_info=depth_map.get_depth_info(seg),
                 )
@@ -172,7 +173,7 @@ class ReflowSequence:
 
     @classmethod
     def from_raw_segments(
-        cls: Type["ReflowSequence"],
+        cls: type["ReflowSequence"],
         segments: Sequence[RawSegment],
         root_segment: BaseSegment,
         config: FluffConfig,
@@ -206,7 +207,7 @@ class ReflowSequence:
 
     @classmethod
     def from_root(
-        cls: Type["ReflowSequence"], root_segment: BaseSegment, config: FluffConfig
+        cls: type["ReflowSequence"], root_segment: BaseSegment, config: FluffConfig
     ) -> "ReflowSequence":
         """Generate a sequence from a root segment.
 
@@ -226,7 +227,7 @@ class ReflowSequence:
 
     @classmethod
     def from_around_target(
-        cls: Type["ReflowSequence"],
+        cls: type["ReflowSequence"],
         target_segment: BaseSegment,
         root_segment: BaseSegment,
         config: FluffConfig,
@@ -353,7 +354,7 @@ class ReflowSequence:
         # the target.
         self.depth_map.copy_depth_info(target, insertion)
         new_block = ReflowBlock.from_config(
-            segments=[insertion],
+            segments=(insertion,),
             config=self.reflow_config,
             depth_info=self.depth_map.get_depth_info(target),
         )
@@ -447,7 +448,7 @@ class ReflowSequence:
 
     def _iter_points_with_constraints(
         self,
-    ) -> Iterator[Tuple[ReflowPoint, Optional[ReflowBlock], Optional[ReflowBlock]]]:
+    ) -> Iterator[tuple[ReflowPoint, Optional[ReflowBlock], Optional[ReflowBlock]]]:
         for idx, elem in enumerate(self.elements):
             # Only evaluate points.
             if isinstance(elem, ReflowPoint):
@@ -503,8 +504,8 @@ class ReflowSequence:
             # If filter has been set, optionally unset the returned values.
             if (
                 filter == "inline"
+                # NOTE: We test on the NEW point.
                 if (
-                    # NOTE: We test on the NEW point.
                     any(seg.is_type("newline") for seg in new_point.segments)
                     # Or if it's followed by the end of file
                     or (post and "end_of_file" in post.class_types)
@@ -533,7 +534,9 @@ class ReflowSequence:
             lint_results=lint_results,
         )
 
-    def rebreak(self) -> "ReflowSequence":
+    def rebreak(
+        self, rebreak_type: Literal["lines", "keywords"] = "lines"
+    ) -> "ReflowSequence":
         """Returns a new :obj:`ReflowSequence` corrected line breaks.
 
         This intentionally **does not handle indentation**,
@@ -552,7 +555,16 @@ class ReflowSequence:
             )
 
         # Delegate to the rebreak algorithm
-        elem_buff, lint_results = rebreak_sequence(self.elements, self.root_segment)
+        if rebreak_type == "lines":
+            elem_buff, lint_results = rebreak_sequence(self.elements, self.root_segment)
+        elif rebreak_type == "keywords":
+            elem_buff, lint_results = rebreak_keywords_sequence(
+                self.elements, self.root_segment
+            )
+        else:  # pragma: no cover
+            raise NotImplementedError(
+                f"Rebreak type of `{rebreak_type}` is not supported."
+            )
 
         return ReflowSequence(
             elements=elem_buff,
@@ -591,15 +603,14 @@ class ReflowSequence:
             lint_results=indent_results,
         )
 
-    def break_long_lines(self):
+    def break_long_lines(self) -> "ReflowSequence":
         """Rebreak any remaining long lines in a sequence.
 
         This assumes that reindent() has already been applied.
         """
         if self.lint_results:
             raise NotImplementedError(  # pragma: no cover
-                "break_long_lines cannot currently handle pre-existing "
-                "embodied fixes."
+                "break_long_lines cannot currently handle pre-existing embodied fixes."
             )
 
         single_indent = construct_single_indent(
