@@ -1,16 +1,8 @@
 """Testing utils for rule plugins."""
 
+from collections.abc import Collection
 from glob import glob
-from typing import (
-    Collection,
-    Dict,
-    List,
-    NamedTuple,
-    Optional,
-    Set,
-    Tuple,
-    Union,
-)
+from typing import NamedTuple, Optional, Union
 
 import pytest
 import yaml
@@ -23,11 +15,12 @@ from sqlfluff.core.errors import (
     SQLParseError,
     SQLTemplaterError,
 )
+from sqlfluff.core.helpers.string import split_comma_separated_string
 from sqlfluff.core.rules import BaseRule, get_ruleset
 from sqlfluff.core.types import ConfigMappingType
 
-FixDictType = Dict[str, Union[str, int]]
-ViolationDictType = Dict[str, Union[str, int, bool, List[FixDictType]]]
+FixDictType = dict[str, Union[str, int]]
+ViolationDictType = dict[str, Union[str, int, bool, list[FixDictType]]]
 
 
 class RuleTestCase(NamedTuple):
@@ -37,18 +30,32 @@ class RuleTestCase(NamedTuple):
     desc: Optional[str] = None
     pass_str: Optional[str] = None
     fail_str: Optional[str] = None
-    violations: Optional[Set[ViolationDictType]] = None
+    violations: Optional[set[ViolationDictType]] = None
     fix_str: Optional[str] = None
-    violations_after_fix: Optional[Set[ViolationDictType]] = None
+    violations_after_fix: Optional[set[ViolationDictType]] = None
     configs: Optional[ConfigMappingType] = None
     skip: Optional[str] = None
-    line_numbers: List[int] = []
+    line_numbers: list[int] = []
+
+    def evaluate(self) -> None:
+        """Evaluate the test case.
+
+        NOTE: This method is designed to be run in a pytest context and
+        will call methods such as `pytest.skip()` as part of it's execution.
+        It may not be suitable for other testing contexts.
+        """
+        rules__test_helper(self)
 
 
 def load_test_cases(
     test_cases_path: str,
-) -> Tuple[List[str], List[RuleTestCase]]:
-    """Load rule test cases from YAML files."""
+) -> tuple[list[str], list[RuleTestCase]]:
+    """Load rule test cases from YAML files.
+
+    Args:
+        test_cases_path (str): A glob string specifying the files containing
+            test cases to load.
+    """
     ids = []
     test_cases = []
 
@@ -93,9 +100,26 @@ def assert_rule_fail_in_sql(
     code: str,
     sql: str,
     configs: Optional[ConfigMappingType] = None,
-    line_numbers: Optional[List[int]] = None,
-) -> Tuple[str, List[SQLBaseError]]:
-    """Assert that a given rule does fail on the given sql."""
+    line_numbers: Optional[list[int]] = None,
+) -> tuple[str, list[SQLBaseError]]:
+    """Assert that a given rule does fail on the given sql.
+
+    Args:
+        code (str): The code of the rule to test.
+        sql (str): The SQL text to check against.
+        configs (:obj:`ConfigMappingType`, optional): A config dict
+            object containing any overrides.
+        line_numbers (list of int, optional): The line numbers which
+            we want to test that errors occurred on.
+
+    Returns:
+        Tuple: values(fixed_sql (str), violations (list))
+            fixed_sql (str): The fixed string after linting. Note that for
+                testing purposes, `.lint_string()` is always called with
+                `fix` set to `True`.
+            violations (list of SQLBaseError): the violations found during
+                linting.
+    """
     print("# Asserting Rule Fail in SQL")
     # Set up the config to only use the rule we are testing.
     cfg = _setup_config(code, configs)
@@ -112,10 +136,10 @@ def assert_rule_fail_in_sql(
     ]
     if parse_errors:
         pytest.fail(f"Found the following parse errors in test case: {parse_errors}")
-    lint_errors: List[SQLLintError] = [
+    lint_errors: list[SQLLintError] = [
         v for v in all_violations if isinstance(v, SQLLintError)
     ]
-    if not any(v.rule.code == code for v in lint_errors):
+    if not any(v.rule.code in split_comma_separated_string(code) for v in lint_errors):
         assert linted.tree
         print(f"Parsed File:\n{linted.tree.stringify()}")
         pytest.fail(
@@ -130,8 +154,20 @@ def assert_rule_fail_in_sql(
                     line_numbers, actual_line_numbers
                 )
             )
-    fixed, _ = linted.fix_string()
-    return fixed, linted.violations
+    fixed_sql, _ = linted.fix_string()
+
+    # Check that if it has made changes that this rule has set
+    # `is_fix_compatible` appropriately.
+    if fixed_sql != sql:
+        assert any(
+            get_rule_from_set(rule, config=cfg).is_fix_compatible
+            for rule in split_comma_separated_string(code)
+        ), (
+            f"Rule {code} returned fixes but does not specify "
+            "'is_fix_compatible = True'."
+        )
+
+    return fixed_sql, linted.violations
 
 
 def assert_rule_pass_in_sql(
@@ -163,7 +199,7 @@ def assert_rule_pass_in_sql(
     # line of code.
     lint_result = linter.lint_string(sql, config=cfg, fname="<STR>")
     lint_errors = [v for v in lint_result.violations if isinstance(v, SQLLintError)]
-    if any(v.rule.code == code for v in lint_errors):
+    if any(v.rule.code in split_comma_separated_string(code) for v in lint_errors):
         print("Errors Found:")
         for e in lint_result.violations:
             print("    " + repr(e))
@@ -174,7 +210,7 @@ def assert_rule_pass_in_sql(
 
 
 def assert_rule_raises_violations_in_file(
-    rule: str, fpath: str, violations: List[Tuple[int, int]], fluff_config: FluffConfig
+    rule: str, fpath: str, violations: list[tuple[int, int]], fluff_config: FluffConfig
 ) -> None:
     """Assert that a given rule raises given errors in specific positions of a file.
 
@@ -204,7 +240,7 @@ def prep_violations(
 
 
 def assert_violations_before_fix(
-    test_case: RuleTestCase, violations_before_fix: List[SQLBaseError]
+    test_case: RuleTestCase, violations_before_fix: list[SQLBaseError]
 ) -> None:
     """Assert that the given violations are found in the given sql."""
     print("# Asserting Violations Before Fix")
